from peak_utility.number import Ordinal

class FinishingPosition(Ordinal):
    def __new__(cls, value, *, tied: bool = False): ...
    def __bool__(self) -> bool: ...
    @classmethod
    def parse(cls, value: int | str): ...



from horsetalk.quantity import HorsetalkQuantity


class RaceWeight(HorsetalkQuantity):
    """
    A class for representing the weight carried by a horse in a race.

    """

    REGEX = r"(?:(\d+)(?:st|\-))?(?:(\d+)(?:lb)*)?"

    def __str__(self) -> str:
        """
        Returns the weight as a string.
        """
        num = self.to("lb").magnitude
        st, lb = divmod(num, 14)
        return f"{st}st {lb}lb"

    @classmethod
    def _string_arg_handler(cls, parts):
        st, lbs = parts
        return (int(st or 0) * 14 + int(lbs or 0), "lb")



from peak_utility.enumeration.parsing_enum import ParsingEnum  # type: ignore


class RacecourseStyle(ParsingEnum):
    """
    An enumeration representing the style of a racecourse.

    """

    UNKNOWN = 0
    GALLOPING = 1
    STIFF = 2
    TIGHT = 3
    SHARP = TIGHT



from .race_class import RaceClass
from .race_grade import RaceGrade


class RaceLevel:
    """
    A class representing the level of the race.

    """

    def __init__(self, value: str | RaceGrade | RaceClass):
        """
        Initialize a RaceLevel instance.

        Args:
            value: The value to initialize with

        Raises:
            ValueError: if Class 1 is given, but no grade is specified

        """
        if isinstance(value, RaceClass) and value == 1:
            raise ValueError("Class 1 race needs a specified grade")

        self.grade = (
            value
            if isinstance(value, RaceGrade)
            else RaceGrade(
                value if "G" in str(value) or str(value).lower() == "listed" else None
            )
        )
        self.class_ = (
            value
            if isinstance(value, RaceClass)
            else RaceClass(1)
            if self.grade
            else RaceClass(value)
        )

    def __repr__(self):
        return f"<RaceLevel: {repr(self.grade) if self.grade else repr(self.class_)}>"

    def __str__(self):
        return f"({int(self.class_)}) {self.grade}".strip()

    def __hash__(self):
        return hash((self.grade, self.class_))

    def __eq__(self, other):
        return self.grade == other.grade and self.class_ == other.class_

    def __gt__(self, other):
        return self.grade > other.grade or self.class_ > other.class_

    def __lt__(self, other):
        return self.grade < other.grade or self.class_ < other.class_



from peak_utility.enumeration.parsing_enum import ParsingEnum

class Disaster(ParsingEnum):
    FELL: int
    REFUSED: int
    BROUGHT_DOWN: int
    UNSEATED_RIDER: int
    PULLED_UP: int
    SLIPPED_UP: int
    CARRIED_OUT: int
    RAN_OUT: int
    LEFT_AT_START: int
    HIT_RAIL: int
    DISQUALIFIED: int
    REFUSED_TO_RACE: int
    F = FELL
    R = REFUSED
    REF = REFUSED
    B = BROUGHT_DOWN
    BD = BROUGHT_DOWN
    U = UNSEATED_RIDER
    UR = UNSEATED_RIDER
    P = PULLED_UP
    PU = PULLED_UP
    S = SLIPPED_UP
    SU = SLIPPED_UP
    C = CARRIED_OUT
    CO = CARRIED_OUT
    O = RAN_OUT
    RO = RAN_OUT
    L = LEFT_AT_START
    LEFT = LEFT_AT_START
    HR = HIT_RAIL
    D = DISQUALIFIED
    DQ = DISQUALIFIED
    DSQ = DISQUALIFIED
    RR = REFUSED_TO_RACE
    @property
    def is_jumping_error(self): ...
    @property
    def is_behavioural_error(self): ...
    @property
    def is_third_party_error(self): ...



from horsetalk import Disaster as Disaster, FinishingPosition as FinishingPosition, FormBreak as FormBreak

class FormFigures:
    @staticmethod
    def parse(form_figures: str) -> list[FinishingPosition | FormBreak | Disaster]: ...



from .race_class import RaceClass as RaceClass
from .race_grade import RaceGrade as RaceGrade
from _typeshed import Incomplete

class RaceLevel:
    grade: Incomplete
    class_: Incomplete
    def __init__(self, value: str | RaceGrade | RaceClass) -> None: ...
    def __hash__(self): ...
    def __eq__(self, other): ...
    def __gt__(self, other): ...
    def __lt__(self, other): ...



from peak_utility.number import RepresentationalInt  # type: ignore


class RaceClass(RepresentationalInt):
    """
    A class to represent a race's class.

    """

    def __new__(cls, value: str | int):
        """
        Create a RaceClass instance.

        Args:
            value: The value to initialize with

        Raises:
            ValueError: If the value is not valid

        """
        class_value = str(value).lower().replace("class", "").strip()

        if not 1 <= int(class_value) <= 7:
            raise ValueError(f"Class must be between 1 and 7, not {value}")

        return super().__new__(cls, int(class_value))

    def __repr__(self) -> str:
        return f"<RaceClass: {int(self)!s}>"

    def __str__(self) -> str:
        return f"Class {int(self)!s}"

    def __hash__(self):
        return hash(str(self))

    def __eq__(self, other):
        if isinstance(other, RaceClass) or (
            isinstance(other, int) and not isinstance(other, RepresentationalInt)
        ):
            return super().__eq__(other)

        return False

    def __ne__(self, other):
        if isinstance(other, RaceClass) or (
            isinstance(other, int) and not isinstance(other, RepresentationalInt)
        ):
            return super().__ne__(other)

        return True

    def __lt__(self, other):
        return super().__gt__(other)

    def __gt__(self, other):
        return super().__lt__(other)



import re

from pint import UnitRegistry

ureg = UnitRegistry(fmt_locale="en_GB")
ureg.define("hand = 4 * inch = hh")
ureg.define("furlong = 0.125 * mile = f")
ureg.define("@alias yard = y")
Q_ = ureg.Quantity


class HorsetalkQuantity(Q_):  # type: ignore
    REGEX = r"(\d+(?:\.\d+)?\D+)"

    def __new__(cls, *args, **kwargs):
        """
        Initializes a HorsetalkQuantity object.

        Args:
            *args: Variable length argument list.
            **kwargs: Arbitrary keyword arguments.

        Returns:
            A HorsetalkQuantity object.
        """
        if args and isinstance(args[0], str):
            arg = args[0].replace(",", "")
            if not re.fullmatch(cls.REGEX, arg):
                raise AttributeError(
                    f"Invalid {cls.__name__.lower()} string: {args[0]}"
                )

            parts = re.match(cls.REGEX, arg).groups()
            args = cls._string_arg_handler(parts)

        if not args:
            args = next(iter(kwargs.items()), None)[::-1]

        instance = Q_.__new__(Q_, *args)
        instance.__class__ = cls
        return instance

    def __repr__(self) -> str:
        """
        Returns:
            A representation of a HorsetalkQuantity object.
        """
        return f"<{self.__class__.__name__}: {self!s}>"

    def __getattr__(self, attr):
        return self.to(attr).magnitude

    @classmethod
    def _string_arg_handler(cls, parts):
        return parts



from peak_utility.enumeration.parsing_enum import ParsingEnum

class RaceDesignation(ParsingEnum):
    HANDICAP: int
    CONDITIONS: int
    MAIDEN: int
    AUCTION: int
    CLAIMER: int
    SELLER: int
    STAKES: int
    HCAP = HANDICAP
    AU = AUCTION
    CL = CLAIMER
    M = MAIDEN
    S = SELLER
    STKS = STAKES
    CLAIMING = CLAIMER
    SELLING = SELLER
    NURSERY = HANDICAP



from peak_utility.number import RepresentationalInt

class RaceClass(RepresentationalInt):
    def __new__(cls, value: str | int): ...
    def __hash__(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __lt__(self, other): ...
    def __gt__(self, other): ...



from peak_utility.enumeration.parsing_enum import ParsingEnum

class Headgear(ParsingEnum):
    HOOD: int
    BLINKERS: int
    CHEEKPIECES: int
    TONGUE_TIE: int
    VISOR: int
    EYE_HOOD: int
    EYE_COVER: int
    EYE_SHIELD: int
    H = HOOD
    B = BLINKERS
    BL = BLINKERS
    C = CHEEKPIECES
    P = CHEEKPIECES
    T = TONGUE_TIE
    TT = TONGUE_TIE
    V = VISOR
    E = EYE_HOOD
    EC = EYE_COVER
    ES = EYE_SHIELD



from .hemisphere import Hemisphere as Hemisphere
from pendulum import DateTime as DateTime, Interval as Interval
from typing import Self

class HorseAge:
    def __init__(self, official_age: int | None = None, *, foaling_date: DateTime | None = None, birth_year: int | None = None, context_date: DateTime | None = None, hemisphere: Hemisphere = ...) -> None: ...
    @property
    def official(self) -> Interval: ...
    @property
    def actual(self) -> Interval: ...
    @property
    def date_of_birth(self) -> DateTime: ...
    @property
    def year_of_birth(self) -> int: ...
    def at(self, date: DateTime) -> Self: ...



from enum import Enum


class FormBreak(Enum):
    """
    An enumeration that represents a break in a horse's form.

    """

    YEAR = "-"
    SEASON = "/"

    def __str__(self):
        return self.value



import pendulum
from .disaster import Disaster as Disaster
from .finishing_position import FinishingPosition as FinishingPosition
from .horselength import Horselength as Horselength
from .outcome import Outcome as Outcome
from _typeshed import Incomplete

class RacePerformance:
    comments: Incomplete
    outcome: Incomplete
    official_position: Incomplete
    beaten_distance: Incomplete
    time: Incomplete
    def __init__(self, outcome: str | int | Disaster | FinishingPosition | Outcome, *, official_position: str | int | FinishingPosition | None = None, beaten_distance: str | int | Horselength | None = None, time: pendulum.Duration | None = None, comments: str | None = None) -> None: ...
    def __lt__(self, other): ...
    @property
    def is_completion(self) -> bool: ...
    @property
    def is_official_win(self) -> bool: ...
    @property
    def is_win(self) -> bool: ...



from .sex import Sex as Sex
from peak_utility.enumeration.parsing_enum import ParsingEnum

class Gender(ParsingEnum):
    FOAL: int
    YEARLING: int
    COLT: int
    FILLY: int
    STALLION: int
    MARE: int
    GELDING: int
    RIG: int
    C = COLT
    F = FILLY
    S = STALLION
    M = MARE
    G = GELDING
    R = RIG
    FOALS = FOAL
    YEARLINGS = YEARLING
    COLTS = COLT
    FILLIES = FILLY
    STALLIONS = STALLION
    MARES = MARE
    GELDINGS = GELDING
    RIGS = RIG
    @property
    def sex(self): ...
    @staticmethod
    def determine(official_age: int, sex: Sex | None = None, **kwargs): ...



from _typeshed import Incomplete
from peak_utility.enumeration.parsing_enum import ParsingEnum

class Silks:
    description: str
    class Colour(ParsingEnum):
        BEIGE: int
        BLACK: int
        BROWN: int
        DARK_BLUE: int
        DARK_GREEN: int
        EMERALD_GREEN: int
        GREEN: int
        GREY: int
        LIGHT_BLUE: int
        LIGHT_GREEN: int
        MAROON: int
        MAUVE: int
        ORANGE: int
        PINK: int
        PURPLE: int
        RED: int
        ROYAL_BLUE: int
        WHITE: int
        YELLOW: int
        @staticmethod
        def phrases(): ...
    class Pattern(ParsingEnum):
        ARMLETS: int
        ARMLET = ARMLETS
        BRACES: int
        CHECK: int
        CHECKED = CHECK
        CHEVRON: int
        CHEVRONS: int
        CROSS_BELTS: int
        CROSS_OF_LORRAINE: int
        CROSS_SASHES = CROSS_BELTS
        CUFFS: int
        DIAMOND: int
        DIAMONDS: int
        DIABOLO: int
        DISC: int
        EPAULETS: int
        HALVED: int
        HOLLOW_BOX: int
        HOOP: int
        HOOPS: int
        HOOPED = HOOPS
        INVERTED_TRIANGLE: int
        LARGE_SPOTS: int
        PANEL: int
        PLAIN: int
        QUARTERED: int
        SASH: int
        SEAMS: int
        SPOTS: int
        STAR: int
        STARS: int
        STRIPE: int
        STRIPES: int
        STRIPED = STRIPES
        TRIPLE_DIAMOND: int
        @staticmethod
        def phrases(): ...
        @classmethod
        def body_only(cls): ...
    class Element:
        primary: Incomplete
        secondary: Incomplete
        pattern: Incomplete
        def __init__(self, primary: Silks.Colour, secondary: Silks.Colour | None = None, pattern: Silks.Pattern | None = None) -> None: ...
        def __eq__(self, other): ...
    @classmethod
    def parse(cls, description: str) -> Silks: ...
    @property
    def body(self) -> Silks.Element: ...
    @property
    def cap(self) -> Silks.Element: ...
    @property
    def sleeves(self) -> Silks.Element: ...



from peak_utility.enumeration.parsing_enum import ParsingEnum

class RacecourseShape(ParsingEnum):
    UNKNOWN: int
    STRAIGHT: int
    HORSESHOE: int
    TRIANGLE: int
    OVAL: int
    PEAR: int
    CIRCLE: int
    ROUND = OVAL
    TRIANGULAR = TRIANGLE



from .age_category import AgeCategory as AgeCategory
from .gender import Gender as Gender
from .horse_experience_level import HorseExperienceLevel as HorseExperienceLevel
from .jump_category import JumpCategory as JumpCategory
from .race_designation import RaceDesignation as RaceDesignation

class RaceTitle:
    @classmethod
    def parse(cls, title: str) -> dict: ...



import re


class AgeRestriction:
    REGEX = (
        r"(\d{1,2})[-]?(\d{1,2})?\s?y(?:ear)?(?:\s|-)?o(?:ld(?:s)?)?(?:\sonly)?(\+)?"
    )

    def __init__(self, string: str | None):
        """
        Initialize an AgeRestriction instance.

        Args:
            age_restriction_string: The age restriction string.

        Returns:
            An AgeRestriction instance.
        """
        if not string or not (matches := re.search(self.REGEX, string)):
            self.minimum = None
            self.maximum = None
        else:
            groups = matches.groups()
            mini, maxi, plus = list(groups) + [None] * (3 - len(groups))

            self.minimum = int(mini)
            self.maximum = None if plus else int(maxi or mini)

    def __repr__(self):
        """
        Returns the age restriction as a repr.

        Returns:
            The age restriction as a repr.
        """
        return f"<AgeRestriction: {self!s}>"

    def __str__(self):
        """
        Returns the age restriction as a string.

        Returns:
            The age restriction as a string.
        """
        if self.minimum == self.maximum:
            return f"{self.minimum}yo"

        if self.maximum:
            return f"{self.minimum}-{self.maximum}yo"

        return f"{self.minimum}yo+"



from .hemisphere import Hemisphere as Hemisphere
from enum import Enum

class Country(Enum):
    def __new__(cls, country_name, hemisphere): ...
    ARG = ...
    AUS = ...
    AUT = ...
    AZE = ...
    BHR = ...
    BAR = ...
    BEL = ...
    BOS = ...
    BRZ = ...
    BUL = ...
    CAN = ...
    CHI = ...
    CHN = ...
    COL = ...
    CRO = ...
    CYP = ...
    CZE = ...
    DEN = ...
    DOM = ...
    ECU = ...
    FIN = ...
    FR = ...
    GER = ...
    GB = ...
    GR = ...
    HER = ...
    HUN = ...
    IND = ...
    IRE = ...
    IRN = ...
    ITY = ...
    JAM = ...
    JPN = ...
    KEN = ...
    KOR = ...
    LEB = ...
    LTU = ...
    LUX = ...
    MAL = ...
    MEX = ...
    MOR = ...
    HOL = ...
    NZ = ...
    NOR = ...
    OM = ...
    PAK = ...
    PRY = ...
    PER = ...
    PHI = ...
    POL = ...
    POR = ...
    PR = ...
    QA = ...
    RUM = ...
    RUS = ...
    KSA = ...
    SER = ...
    SVK = ...
    SVN = ...
    SAF = ...
    SPA = ...
    SWE = ...
    SWI = ...
    SY = ...
    TRI = ...
    TUN = ...
    TUR = ...
    UAE = ...
    USA = ...
    URU = ...
    UZB = ...
    VEN = ...
    ZIM = ...



from enum import Enum

from .hemisphere import Hemisphere


class Country(Enum):
    """
    An enumeration representing a country.

    """

    def __new__(cls, country_name, hemisphere):
        obj = object.__new__(cls)
        obj._value_ = country_name
        obj.hemisphere = hemisphere
        return obj

    ARG = ("Argentina", Hemisphere.SOUTH)
    AUS = ("Australia", Hemisphere.SOUTH)
    AUT = ("Austria", Hemisphere.NORTH)
    AZE = ("Azerbaijan", Hemisphere.NORTH)
    BHR = ("Bahrain", Hemisphere.NORTH)
    BAR = ("Barbados", Hemisphere.NORTH)
    BEL = ("Belgium", Hemisphere.NORTH)
    BOS = ("Bosnia", Hemisphere.NORTH)
    BRZ = ("Brazil", Hemisphere.SOUTH)
    BUL = ("Bulgaria", Hemisphere.NORTH)
    CAN = ("Canada", Hemisphere.NORTH)
    CHI = ("Chile", Hemisphere.SOUTH)
    CHN = ("China", Hemisphere.NORTH)
    COL = ("Colombia", Hemisphere.SOUTH)
    CRO = ("Croatia", Hemisphere.NORTH)
    CYP = ("Cyprus", Hemisphere.NORTH)
    CZE = ("Czech Republic", Hemisphere.NORTH)
    DEN = ("Denmark", Hemisphere.NORTH)
    DOM = ("Dominican Republic", Hemisphere.NORTH)
    ECU = ("Ecuador", Hemisphere.SOUTH)
    FIN = ("Finland", Hemisphere.NORTH)
    FR = ("France", Hemisphere.NORTH)
    GER = ("Germany", Hemisphere.NORTH)
    GB = ("Great Britain", Hemisphere.NORTH)
    GR = ("Greece", Hemisphere.NORTH)
    HER = ("Herzegovina", Hemisphere.NORTH)
    HUN = ("Hungary", Hemisphere.NORTH)
    IND = ("India", Hemisphere.NORTH)
    IRE = ("Ireland", Hemisphere.NORTH)
    IRN = ("Iran", Hemisphere.NORTH)
    ITY = ("Italy", Hemisphere.NORTH)
    JAM = ("Jamaica", Hemisphere.NORTH)
    JPN = ("Japan", Hemisphere.NORTH)
    KEN = ("Kenya", Hemisphere.NORTH)
    KOR = ("Korea", Hemisphere.NORTH)
    LEB = ("Lebanon", Hemisphere.NORTH)
    LTU = ("Lithuania", Hemisphere.NORTH)
    LUX = ("Luxembourg", Hemisphere.NORTH)
    MAL = ("Malaysia", Hemisphere.NORTH)
    MEX = ("Mexico", Hemisphere.NORTH)
    MOR = ("Morocco", Hemisphere.NORTH)
    HOL = ("Netherlands", Hemisphere.NORTH)
    NZ = ("New Zealand", Hemisphere.SOUTH)
    NOR = ("Norway", Hemisphere.NORTH)
    OM = ("Oman", Hemisphere.NORTH)
    PAK = ("Pakistan", Hemisphere.NORTH)
    PRY = ("Paraguay", Hemisphere.SOUTH)
    PER = ("Peru", Hemisphere.SOUTH)
    PHI = ("Philippines", Hemisphere.NORTH)
    POL = ("Poland", Hemisphere.NORTH)
    POR = ("Portugal", Hemisphere.NORTH)
    PR = ("Puerto Rico", Hemisphere.NORTH)
    QA = ("Qatar", Hemisphere.NORTH)
    RUM = ("Romania", Hemisphere.NORTH)
    RUS = ("Russia", Hemisphere.NORTH)
    KSA = ("Saudi Arabia", Hemisphere.NORTH)
    SER = ("Serbia", Hemisphere.NORTH)
    SVK = ("Slovakia", Hemisphere.NORTH)
    SVN = ("Slovenia", Hemisphere.NORTH)
    SAF = ("South Africa", Hemisphere.SOUTH)
    SPA = ("Spain", Hemisphere.NORTH)
    SWE = ("Sweden", Hemisphere.NORTH)
    SWI = ("Switzerland", Hemisphere.NORTH)
    SY = ("Syria", Hemisphere.NORTH)
    TRI = ("Trinidad and Tobago", Hemisphere.NORTH)
    TUN = ("Tunisia", Hemisphere.NORTH)
    TUR = ("Turkey", Hemisphere.NORTH)
    UAE = ("United Arab Emirates", Hemisphere.NORTH)
    USA = ("United States of America", Hemisphere.NORTH)
    URU = ("Uruguay", Hemisphere.SOUTH)
    UZB = ("Uzbekistan", Hemisphere.NORTH)
    VEN = ("Venezuela", Hemisphere.NORTH)
    ZIM = ("Zimbabwe", Hemisphere.SOUTH)

    def __str__(self):
        return self.value



from .aw_going_description import AWGoingDescription
from .dirt_going_description import DirtGoingDescription
from .going_description import GoingDescription
from .surface import Surface
from .turf_going_description import TurfGoingDescription


class Going:
    """
    A class to represent a going.
    """

    Scales = (TurfGoingDescription, AWGoingDescription, DirtGoingDescription)

    def __init__(self, description: str, reading: float | None = None):
        """
        Initialize a Going instance.

        Args:
            description: The description of the going.
            reading: The reading of the going stick.
        """
        self.description = description.replace(" (", ", ").replace(")", "")
        self.reading = reading

        assert (self.primary and self.secondary) or self.primary

    def __hash__(self):
        """
        The hash of the going.

        Returns:
            The hash of the going.
        """
        return hash((self.primary, self.secondary, self.reading))

    def __repr__(self):
        """
        The repr of the going.

        Returns:
            The repr of the going.
        """
        return f"<Going({self.primary!r}, {self.secondary!r})>"

    def __str__(self):
        """
        The string representation of the going.

        Returns:
            The string representation of the going.
        """
        primary_str = self.primary.name.title()
        secondary_str = self.secondary.name.title() if self.secondary else ""
        return f"{primary_str}{', ' + secondary_str + ' in places' if secondary_str else ''}".replace(
            "_", " "
        ).replace(" To", " to")

    def __eq__(self, other: object) -> bool:
        """
        Determine if two goings are equal.

        Args:
            other: The other going.

        Returns:
            True if the goings are equal, False otherwise.
        """

        return isinstance(other, Going) and (
            self.primary == other.primary
            and self.secondary == other.secondary
            and self.reading == other.reading
        )

    @property
    def primary(self) -> GoingDescription:
        """
        The primary property of the going.

        Returns:
            A value selected from the appropriate going scale.
        """
        key = self._description_parts[0]
        return Going._lookup(key)

    @property
    def secondary(self) -> GoingDescription | None:
        """
        The secondary or 'in places' property of the going.

        Returns:
            A value selected from the appropriate going scale.
        """
        key = self._description_parts[1]
        return Going._lookup(key) if key else None

    @property
    def surface(self) -> Surface:
        """
        The surface implied by the going description.

        Returns:
            The Surface enum member implied by the going description.
        """
        return self.primary.surface

    @property
    def value(self) -> float:
        """
        A numerical value for the going.

        Returns:
            The value of the going.
        """
        return (
            self.primary.value
            if self.secondary is None
            else (self.primary.value + self.secondary.value) / 2
        )

    @property
    def _description_parts(self) -> list[str]:
        """
        The parts of the description.

        Returns:
            The parts of the description.
        """
        texts = self.description.upper().replace(" IN PLACES", "").split(", ")

        if len(texts) == 2:
            if texts[0] == texts[1]:
                raise ValueError("Primary and secondary going description cannot match")
            return texts

        return [*texts, ""]

    @classmethod
    def _lookup(cls, key: str) -> GoingDescription:
        """
        Lookup a value in the appropriate going scale.

        Args:
            key: The key to lookup.

        Returns:
            A value selected from the appropriate going scale.

        Raises:
            ValueError: If the key is not found in any of the going scales.
        """
        for scale in Going.Scales:
            try:
                return scale[key]  # type: ignore
            except KeyError:
                pass

        raise ValueError(f"Unknown going description: {key}")

    @staticmethod
    def multiparse(description: str) -> dict[str, "Going"]:
        """
        Sometimes goings are given for multiple courses, e.g. turf and all weather, chase and hurdle in the same string.
        This will parse these into a dict of Going objects, keyed by an identifier for each course

        Args:
            going: The going string to parse.

        Returns:
            A dict of Going objects.
        """
        opposites = {
            "hurdle": "chase",
            "chase": "hurdle",
            "inner": "outer",
            "outer": "inner",
            "old": "new",
            "new": "old",
            "straight": "round",
            "round": "straight",
            "flat": "aw",
            "aw": "flat",
        }

        if not any(x in description.lower() for x in opposites):
            return {"default": Going(description)}

        clauses = (
            description.lower()
            .replace(")", "")
            .replace("(", ",")
            .replace("hurdles", "hurdle")
            .split(",")
        )
        for i, clause in enumerate(clauses):
            if not any(x in clause for x in opposites) and "in places" in clause:
                clauses[i - 1] = clauses[i - 1].strip() + ", " + clause
                clauses[i] = ""
        clauses = [x for x in clauses if x]

        identifier = next(x for x in opposites if x in description.lower())

        def strip_clause(x, y):
            return (
                x.replace("on", "")
                .replace("course", "")
                .replace(":", "")
                .replace(y, "")
                .strip()
            )

        def reconstructed_going_description(identifier: str, clauses: list[str]) -> str:
            containing_clause = next((x for x in clauses if identifier in x), "")
            return (
                f"{clauses[0]}, {strip_clause(containing_clause, identifier)} in places"
                if "in places" in containing_clause and "," not in containing_clause
                else strip_clause(containing_clause, identifier)
                if containing_clause
                else clauses[0]
            )

        return {
            i: Going(reconstructed_going_description(i, clauses))
            for i in [identifier, opposites[identifier]]
        }



from peak_utility.enumeration.parsing_enum import ParsingEnum  # type: ignore


class JockeyExperienceLevel(ParsingEnum):
    """
    An enumeration that represents a jockey's experience level.

    """

    AMATEUR = 1
    CONDITIONAL = 2
    APPRENTICE = 3
    PROFESSIONAL = 4

    # Plurals
    AMATEURS = AMATEUR
    CONDITIONALS = CONDITIONAL
    APPRENTICES = APPRENTICE
    PROFESSIONALS = PROFESSIONAL



from .going_description import GoingDescription as GoingDescription

class DirtGoingDescription(GoingDescription):
    MUDDY: int
    SLOW: int
    SLOPPY: float
    GOOD: int
    WET_FAST: float
    SEALED: int
    FAST: int



from peak_utility.enumeration.parsing_enum import ParsingEnum

class RacingCode(ParsingEnum):
    FLAT: int
    NATIONAL_HUNT: int
    POINT_TO_POINT: int
    F = FLAT
    N = NATIONAL_HUNT
    NH = NATIONAL_HUNT
    P = POINT_TO_POINT
    PTP = POINT_TO_POINT
    P2P = POINT_TO_POINT



from peak_utility.enumeration.parsing_enum import ParsingEnum  # type: ignore


class RacecourseShape(ParsingEnum):
    """
    An enumeration representing the shape of a racecourse.

    """

    UNKNOWN = 0
    STRAIGHT = 1
    HORSESHOE = 2
    TRIANGLE = 3
    OVAL = 4
    PEAR = 5
    CIRCLE = 6

    # Alternatives
    ROUND = OVAL
    TRIANGULAR = TRIANGLE



from peak_utility.enumeration.parsing_enum import ParsingEnum

class Hemisphere(ParsingEnum):
    NORTHERN: int
    SOUTHERN: int
    N = NORTHERN
    S = SOUTHERN
    NORTH = NORTHERN
    SOUTH = SOUTHERN



from peak_utility.enumeration.parsing_enum import ParsingEnum  # type: ignore


class Handedness(ParsingEnum):
    """
    An enumeration representing the handedness of a racecourse.

    """

    UNKNOWN = 0
    LEFT = 1
    RIGHT = 2
    BOTH = 3

    # Alternatives
    NEITHER = UNKNOWN
    L = LEFT
    R = RIGHT
    LR = BOTH
    LH = LEFT
    RH = RIGHT



from peak_utility.enumeration.parsing_enum import ParsingEnum

class HorseExperienceLevel(ParsingEnum):
    MAIDEN: int
    NOVICE: int
    BEGINNER: int
    NOVICES = NOVICE
    BEGINNERS = BEGINNER
